package eu.maiora.db

import com.fasterxml.jackson.databind.deser.impl.CreatorCandidate.Param
import eu.maiora.model.Accounts
import eu.maiora.model.Parametri
import eu.maiora.model.Tessere
import kotlinx.coroutines.Dispatchers
import org.jetbrains.exposed.dao.IntEntity
import org.jetbrains.exposed.dao.IntEntityClass
import org.jetbrains.exposed.dao.id.EntityID
import org.jetbrains.exposed.dao.id.IdTable
import org.jetbrains.exposed.sql.Transaction
import org.jetbrains.exposed.sql.transactions.experimental.newSuspendedTransaction

object AccountsTable : IdTable<Int>("accounts"){
    override val id = integer("id").entityId()
    val username = varchar("username", 255)
    val password = varchar("password", 255)

    override val primaryKey = PrimaryKey(id)
}

object ParametriTable : IdTable<Int>("parametri"){
    override val id = integer("id").entityId()
    val chiave = varchar("chiave", 255)
    val valore = varchar("valore", 255)

    override val primaryKey = PrimaryKey(id)
}

object TessereTable : IdTable<Long>("view_tessere_api"){
    override val id = long("id").entityId()
    val idUtente = long("id_utente")
    val codiceFiscale = varchar("codice_fiscale", 255)
    val numero = varchar("numero", 255)
    val saldo = double("saldo")
    val punti = integer("punti")

    override val primaryKey = PrimaryKey(id)
}

class AccountsDAO(id: EntityID<Int>) :IntEntity(id) {
    companion object : IntEntityClass<AccountsDAO>(AccountsTable)

    var username by AccountsTable.username
    var password by AccountsTable.password
}

class ParametriDAO(id: EntityID<Int>) :IntEntity(id) {
    companion object : IntEntityClass<ParametriDAO>(ParametriTable)

    var chiave by ParametriTable.chiave
    var valore by ParametriTable.valore
}

class TessereDao(id: EntityID<Long>) :LongEntity(id) {
    companion object : LongEntityClass<TessereDao>(TessereTable)

    var idUtente by TessereTable.idUtente
    var codiceFiscale by TessereTable.codiceFiscale
    var numero by TessereTable.numero
    var saldo by TessereTable.saldo
    var punti by TessereTable.punti
}


fun accountsDaoToModel(dao: AccountsDAO) = Accounts(
    dao.id.value,
    dao.username,
    dao.password
)

fun parametriDaoToModel(dao: ParametriDAO) = Parametri(
    dao.id.value,
    dao.chiave,
    dao.valore
)

fun tessereDaoToModel(dao: TessereDao) = Tessere(
    dao.id.value,
    dao.idUtente,
    dao.codiceFiscale,
    dao.numero,
    dao.saldo,
    dao.punti
)


suspend fun <T> suspendTransaction(block: Transaction.() -> T): T =
    newSuspendedTransaction(Dispatchers.IO, statement = block)